#!/bin/sh
# Script to automate installation of OpenSSH on Zauruz 5K series Linux PDA
# by maldoror@worldnet.att.net
# Must be run as root!
#
# V1.0		2/25/02
#	Initial release
# V1.1		2/28/02
#	Changes:
#	Fixed warning-banner copy problem
#	Added symlink for "/usr/local/bin/ssh" to fix sftp dependency
#	Added backup of config files prior to run. Lack of free space may lead to corruption of "/etc/inetd.conf" and "/etc/passwd", so we now back them up before doing anything.
#	Added removal of existing ssh binaries and symbolic links prior to installation.  This fixes problems when re-installing or upgrading.
#	Removes old ssh entries in /etc/inetd.conf to prevent problems cause by re-installing to a different destination
#	Stripped binaries to reduce size
#	Grep ssh out of netstat to verify restart of inetd at end of script
#	Added paranoid file permissioning of SSH files just in case anyone uses their Zaurus as a multi-user Unix server
#	Fixed bug that caused all three user keys to be RSA keys for SSH1
# V1.2		3/5/02
#	Changes:
#	Minor stdout redirection problem fixed.  This was potentially whacking the edited /etc/password file so that it only contains a root account entry.  This didn't pose a functional problem.
# V1.2.1	3/6/02
#	Changes:
#	Added check to make sure that the script is being run from the same directory as the extracted binary files (thanks to rwarner1@directvinternet.com).
#	Added symbolic link from /usr/local/etc/sshd_config to /etc/sshd_config.  This way, if you launch sshd from the command line, it will find the config file in the compile-in default location of /usr/local/etc/sshd_config (thanks to rwarner1@directvinternet.com).
# V1.2.2	3/7/02
#	Changes:
#	Updated binaries to OpenSSH 3.1-p1
#	Moved ssh config files from /etc to /etc/ssh
#	Made changes to reflect that /usr/local references are no longer required.  Openssh 3.1-p1 was properly compiled with /usr and /etc as the appropriate paths.
# V1.3		4/1/02
#	Changes:
#	Fix some stupid stuff like reverting user keys to default names
#	Fix bug related to config files not being referenced correctly in the new directory /etc/ssh
#	Moved search path update from .bashrc to .profile (more appropriate but no functional difference)

# Select install destination
if [ -z $1 ]; then
	echo "Usage: $0 [m=main memory | c=compact flash | s=sd ram]"
	echo "Note: Preferred installation is to SD RAM since"
	echo "      CF may be occupied by a network adapter."
	exit 1
fi

case "$1" in
"m")
	PREFIX=/usr;;
"c")
	PREFIX=/mnt/cf;;
"s")
	PREFIX=/mnt/card;;
*)
	exit 1;;
esac

# Verify that we are running this script from the same directory as the ssh binaries.  We only check for sshd.
if ! [ -f ./sshd ]; then
	echo "Please run this script from the same directory"
	echo "as the bundled binary files."
	exit 1
fi

echo "--- Installing OpenSSH ---"
mkdir /home/root/.ssh 1>/dev/null 2>&1
mkdir /etc/ssh 1>/dev/null 2>&1
if [ $1 = c ]; then
	mkdir /mnt/cf/bin 1>/dev/null 2>&1
	mkdir /mnt/cf/sbin 1>/dev/null 2>&1
fi

if [ $1 = s ]; then
	mkdir /mnt/card/bin 1>/dev/null 2>&1
	mkdir /mnt/card/sbin 1>/dev/null 2>&1
fi

echo ""
echo "----- Backing up original config files"
echo "       /etc/inetd  --> /etc/inetd.backup"
cp /etc/inetd /etc/inetd.backup 1>/dev/null 2>&1
echo "       /etc/passwd --> /etc/passwd.backup"
cp /etc/passwd /etc/passwd.backup 1>/dev/null 2>&1

echo ""
echo "----- Deleting old ssh binaries and links"
echo "      (preserving old conf files and keys)"
rm -f /usr/sbin/sshd 1>/dev/null 2>&1
rm -f /usr/sbin/sftp-server 1>/dev/null 2>&1
rm -f /usr/bin/ssh 1>/dev/null 2>&1
rm -f /usr/bin/ssh-keygen 1>/dev/null 2>&1
rm -f /usr/bin/ssh-keygen 1>/dev/null 2>&1
rm -f /usr/bin/ssh-agent 1>/dev/null 2>&1
rm -f /usr/bin/ssh-add 1>/dev/null 2>&1
rm -f /usr/bin/sftp 1>/dev/null 2>&1
rm -f /usr/bin/scp 1>/dev/null 2>&1

# Check to see if sshd is already defined in inetd.conf.  If so, then skip next step.
echo ""
echo "----- Adding sshd to /etc/inetd.conf"
grep -v ^ssh /etc/inetd.conf 1>/tmp/inetd.conf 2>/dev/null
echo "#sshd support added below">>/tmp/inetd.conf
	case "$1" in
		"m")
			echo "ssh	stream	tcp	nowait	root	/usr/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
		"c")
			echo "ssh	stream	tcp	nowait	root	/mnt/cf/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
		"s")
			echo "ssh	stream	tcp	nowait	root	/mnt/card/sbin/sshd sshd -i -q -f /etc/ssh/sshd_config">>/tmp/inetd.conf;;
	esac
cp /tmp/inetd.conf /etc/inetd.conf 1>/dev/null 2>&1
rm -f /tmp/inetd.conf 1>/dev/null 2>&1

echo ""
echo "----- Copying configuration files"

if [ -f /etc/sshd_config ]; then
	echo "Warning: openssh 3.1 has moved the /etc/sshd_config file into /etc/ssh/sshd_config."
	echo "         Please manually edit the /etc/ssh/sshd_config file to reflect old settings."
	echo "         The old sshd_config can be found at /etc/ssh/sshd_config.old"
	mv -f /etc/sshd_config /etc/ssh/sshd_config.old 1>/dev/null 2>&1
fi

if [ -f /etc/ssh_config ]; then
	echo "Warning: openssh 3.1 has moved the /etc/ssh_config file into /etc/ssh/ssh_config."
	echo "         Please manually edit the /etc/ssh/ssh_config file to reflect old settings."
	echo "         The old ssh_config can be found at /etc/ssh/ssh_config.old"
	mv -f /etc/ssh_config /etc/ssh/ssh_config.old 1>/dev/null 2>&1
fi

if ! [ -f /etc/ssh/sshd_config ]; then
	cp -f sshd_config /etc/ssh 1>/dev/null 2>&1
else
	echo "       /etc/ssh/sshd_config exists"
fi

if ! [ -f /etc/ssh/ssh_config ]; then
	cp -f ssh_config /etc/ssh 1>/dev/null 2>&1
else
	echo "       /etc/ssh/ssh_config exists"
fi

if ! [ -f /home/root/.ssh/ssh_config ]; then
	cp -f ssh_config /home/root/.ssh 1>/dev/null 2>&1
else
	echo "       /home/root/.ssh/ssh_config exists"
fi

#Copy the banner file, but move an existing one if it exists.
if [ -f /etc/ssh_banner ]; then
	mv /etc/ssh_banner /etc/ssh 1>/dev/null 2>&1
else
	cp -f ssh_banner /etc/ssh 1>/dev/null 2>&1
fi

echo ""
echo "----- Copying server programs"
cp -f sshd $PREFIX/sbin 1>/dev/null 2>&1
cp -f sftp-server $PREFIX/sbin 1>/dev/null 2>&1

echo ""
echo "----- Copying client programs"
cp -f ssh $PREFIX/bin 1>/dev/null 2>&1
cp -f ssh-keygen $PREFIX/bin 1>/dev/null 2>&1
cp -f ssh-agent $PREFIX/bin 1>/dev/null 2>&1
cp -f sftp $PREFIX/bin 1>/dev/null 2>&1
cp -f ssh-keyscan $PREFIX/bin 1>/dev/null 2>&1
cp -f scp $PREFIX/bin 1>/dev/null 2>&1
cp -f ssh-add $PREFIX/bin 1>/dev/null 2>&1

# Adding shared library
echo ""
echo "----- Copying shared libraries"
mkdir $PREFIX/lib 1>/dev/null 2>&1
cp -f libz.so.1.1.3 $PREFIX/lib 1>/dev/null 2>&1
ln -s $PREFIX/lib/libz.so.1.1.3 /usr/lib/libz.so.1 1>/dev/null 2>&1
ln -s $PREFIX/lib/libz.so.1.1.3 /usr/lib/libz.so.1.1.3 1>/dev/null 2>&1

echo ""
echo "----- Generating Server Keys"

# Move existing openssh <3.1 keys into the /etc/ssh directory if they exist.
# This is used when upgrading openssh.
mv -f ssh_*_key* ./ssh 1>/dev/null 2>&1

if ! [ -f "/etc/ssh/ssh_host_key" ]; then
	echo "------- Generating RSA key for SSH v1"
	./ssh-keygen -N "" -q -t rsa1 -f /etc/ssh/ssh_host_key  1>/dev/null 2>&1
else
	echo "      /etc/ssh/ssh_host_key exists"
fi

if ! [ -f "/etc/ssh_host_rsa_key" ]; then
	echo "------- Generating RSA key for SSH v2"
	./ssh-keygen -N "" -q -t rsa -f /etc/ssh/ssh_host_rsa_key  1>/dev/null 2>&1
else
	echo "      /etc/ssh/ssh_host_rsa_key exists"
fi

if ! [ -f "/etc/ssh_host_dsa_key" ]; then
	echo "------- Generating DSA key for SSH v2"
	./ssh-keygen -N "" -q -t dsa -f /etc/ssh/ssh_host_dsa_key  1>/dev/null 2>&1
else
	echo "      /etc/ssh/ssh_host_dsa_key exists"
fi

echo ""
echo "----- Generating User Keys"

if ! [ -f "/home/root/.ssh/identity" ]; then
	echo "------- Generating RSA key for SSH v1"
	./ssh-keygen -t rsa1 -f /home/root/.ssh/identity
else
	echo "      $HOME/.ssh/identity exists"
fi

if ! [ -f "/home/root/.ssh/id_rsa" ]; then
	echo "------- Generating RSA key for SSH v2"
	./ssh-keygen -t rsa -f /home/root/.ssh/id_rsa
else
	echo "      $HOME/id_rsa exists"
fi
if ! [ -f "/home/root/.ssh/id_dsa" ]; then
	echo "------- Generating DSA key for SSH v2"
	./ssh-keygen -t dsa -f /home/root/.ssh/id_dsa
else
	echo "      $HOME/.ssh/id_dsa exists"
fi

echo ""
echo "----- Installing HTML version of man pages"
mkdir /home/root/Documents/OpenSSH 1>/dev/null 2>&1
cp -f *.htm /home/root/Documents/OpenSSH  1>/dev/null 2>&1
cp -f readme /home/root/Documents/OpenSSH 1>/dev/null 2>&1
cp -f README.Zaurus /home/root/Documents/OpenSSH 1>/dev/null 2>&1

# Update local search path.  This is redundant since we symlink binaries to /usr/bin and /usr/sbin, but it can't hurt
echo ""
echo "----- Updating search path"
case "$1" in
	"c")
		echo "PATH=$PATH:/mnt/cf/bin:/mnt/cf/sbin">>/home/root/.profile;;
	"s")
		echo "PATH=$PATH:/mnt/card/bin:/mnt/card/sbin">>/home/root/.profile;;
esac

echo ""
echo "----- Setting file permissions"
chmod 755 $PREFIX\bin\ssh 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\ssh-keygen 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\ssh-keyscan 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\ssh-agent 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\ssh-add 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\scp 1>/dev/null 2>&1
chmod 755 $PREFIX\bin\sftp 1>/dev/null 2>&1
chmod 755 $PREFIX\sbin\sshd 1>/dev/null 2>&1
chmod 755 $PREFIX\sbin\sftp-server 1>/dev/null 2>&1
chmod 600 /etc/ssh/sshd_config 1>/dev/null 2>&1
chmod 644 /etc/ssh/ssh_config 1>/dev/null 2>&1
chmod 600 /etc/ssh/ssh_host_key 1>/dev/null 2>&1
chmod 600 /etc/ssh/ssh_host_rsa_key 1>/dev/null 2>&1
chmod 600 /etc/ssh/ssh_host_dsa_key 1>/dev/null 2>&1
chmod 644 /etc/ssh/ssh_host_key.pub 1>/dev/null 2>&1
chmod 644 /etc/ssh/ssh_host_rsa_key.pub 1>/dev/null 2>&1
chmod 644 /etc/ssh/ssh_host_dsa_key.pub 1>/dev/null 2>&1
chmod 644 /etc/ssh/ssh_banner 1>/dev/null 2>&1
chmod 600 /home/root/.ssh/ssh_config 1>/dev/null 2>&1
chmod 600 /home/root/.ssh/identity 1>/dev/null 2>&1
chmod 600 /home/root/.ssh/id_rsa 1>/dev/null 2>&1
chmod 600 /home/root/.ssh/id_dsa 1>/dev/null 2>&1
chmod 644 /home/root/.ssh/identity.pub 1>/dev/null 2>&1
chmod 644 /home/root/.ssh/id_rsa.pub 1>/dev/null 2>&1
chmod 644 /home/root/.ssh/id_dsa.pub 1>/dev/null 2>&1

echo ""
echo "----- Creating symbolic links"
ln -s $PREFIX/bin/ssh /usr/bin/ssh 1>/dev/null 2>&1
ln -s $PREFIX/bin/ssh-keygen /usr/bin/ssh-keygen 1>/dev/null 2>&1
ln -s $PREFIX/bin/ssh-keyscan /usr/bin/ssh-keyscan 1>/dev/null 2>&1
ln -s $PREFIX/bin/scp /usr/bin/scp 1>/dev/null 2>&1
ln -s $PREFIX/bin/sftp /usr/bin/sftp 1>/dev/null 2>&1
ln -s $PREFIX/bin/ssh-add /usr/bin/ssh-add 1>/dev/null 2>&1
ln -s $PREFIX/bin/ssh-agent /usr/bin/ssh-agent 1>/dev/null 2>&1
ln -s $PREFIX/sbin/sshd /usr/sbin/sshd 1>/dev/null 2>&1
ln -s $PREFIX/sbin/sftp-server /usr/sbin/sftp-server 1>/dev/null 2>&1

echo ""
echo "----- Fixing path on root entry in /etc/passwd"
grep -v ^root /etc/passwd 1>/tmp/passwd  2>/dev/null
echo "root:x:0:0:root:/home/root:/bin/sh">/tmp/passwd
cp -f /tmp/passwd /etc/passwd 1>/dev/null 2>&1
rm -f /tmp/passwd  1>/dev/null 2>&1

echo ""
echo "----- Please set a root password for the Zaurus"
passwd

echo ""
echo "----- Restarting inetd"
# We pull the process id for the restart of inetd from /var/run/inetd.pid
kill -HUP `cat /var/run/inetd.pid` 1>/dev/null 2>&1

echo ""
echo "----- Installation complete - ssh ready!"
# Do a netstat to verify that sshd is listening on 0.0.0.0
echo ""
netstat -a | grep :ssh
echo ""
